@@ -328,6 +328,18 @@ thunar_device_emit_pre_unmount (ThunarDevice *device,
 
 
 
+/**
+ * thunar_device_get_name:
+ * @device     : a #ThunarDevice instance.
+ *
+ * Returns the name of the @device if the @device
+ * is mounted.
+ *
+ * The caller is responsible to free the returned string
+ * using g_free() when no longer needed.
+ *
+ * Return value: the name of @device if @device is mounted
+ **/
 gchar *
 thunar_device_get_name (const ThunarDevice *device)
 {
@@ -664,9 +676,13 @@ thunar_device_get_root (const ThunarDevice *device)
 
 
 gint
-thunar_device_sort (const ThunarDevice *device1,
-                    const ThunarDevice *device2)
+thunar_device_compare_by_name (const ThunarDevice *device1,
+                               const ThunarDevice *device2)
 {
+  gchar* name1;
+  gchar* name2;
+  gint   result;
+
   _thunar_return_val_if_fail (THUNAR_IS_DEVICE (device1), 0);
   _thunar_return_val_if_fail (THUNAR_IS_DEVICE (device2), 0);
 
@@ -674,8 +690,15 @@ thunar_device_sort (const ThunarDevice *device1,
   if (G_OBJECT_TYPE (device1->device) != G_OBJECT_TYPE (device2->device))
     return G_IS_MOUNT (device1->device) ? 1 : -1;
 
-  /* sort by detect stamp */
-  return device1->stamp > device2->stamp ? 1 : -1;
+  name1 = thunar_device_get_name (device1);
+  name2 = thunar_device_get_name (device2);
+
+  result = g_strcmp0 (name1, name2);
+
+  g_free (name1);
+  g_free (name2);
+
+  return result;
 }
 
 
